@@ -1,5 +1,5 @@
 /*
- * "$Id: snmp.c 10209 2012-01-30 22:19:03Z mike $"
+ * "$Id: snmp.c 10643 2012-10-09 03:02:49Z mike $"
  *
  *   SNMP discovery backend for CUPS.
  *
@@ -251,7 +251,7 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 
 #ifdef AF_INET6
   if ((ipv6 = _cupsSNMPOpen(AF_INET6)) < 0)
-    return (1);
+    perror("DEBUG: Unable to create IPv6 socket");
 #else
   ipv6 = -1;
 #endif /* AF_INET6 */
@@ -1025,6 +1025,11 @@ read_snmp_response(int fd)		/* I - SNMP socket file descriptor */
 	    * Description is the IEEE-1284 device ID...
 	    */
 
+            char *ptr;			/* Pointer into device ID */
+
+            for (ptr = (char *)packet.object_value.string.bytes; *ptr; ptr ++)
+              if (*ptr == '\n')
+                *ptr = ';';		/* A lot of bad printers put a newline */
 	    if (!device->id)
 	      device->id = strdup((char *)packet.object_value.string.bytes);
 
@@ -1066,8 +1071,11 @@ read_snmp_response(int fd)		/* I - SNMP socket file descriptor */
 	  */
 
 	  char	make_model[256];	/* Make and model */
+          char *ptr;			/* Pointer into device ID */
 
-
+          for (ptr = (char *)packet.object_value.string.bytes; *ptr; ptr ++)
+            if (*ptr == '\n')
+              *ptr = ';';		/* A lot of bad printers put a newline */
 	  if (device->id)
 	    free(device->id);
 
@@ -1383,5 +1391,5 @@ update_cache(snmp_cache_t *device,	/* I - Device */
 
 
 /*
- * End of "$Id: snmp.c 10209 2012-01-30 22:19:03Z mike $".
+ * End of "$Id: snmp.c 10643 2012-10-09 03:02:49Z mike $".
  */
