@@ -1,9 +1,9 @@
 /*
- * "$Id: lp.c 10494 2012-05-21 22:39:01Z mike $"
+ * "$Id: lp.c 10830 2013-01-21 13:14:40Z mike $"
  *
  *   "lp" command for CUPS.
  *
- *   Copyright 2007-2012 by Apple Inc.
+ *   Copyright 2007-2013 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -489,6 +489,13 @@ main(int  argc,				/* I - Number of command-line arguments */
 	    break;
 
         case '-' : /* Stop processing options */
+            if (argv[i][2])
+            {
+	      _cupsLangPrintf(stderr, _("%s: Error - unknown option \"%s\"."),
+			      argv[0], argv[i]);
+	      return (1);
+	    }
+
 	    end_options = 1;
 	    break;
 
@@ -724,5 +731,5 @@ set_job_attrs(const char    *command,	/* I - Command name */
 
 
 /*
- * End of "$Id: lp.c 10494 2012-05-21 22:39:01Z mike $".
+ * End of "$Id: lp.c 10830 2013-01-21 13:14:40Z mike $".
  */
